import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

public class DVDMaker extends JFrame implements ActionListener {
	
	public static final String installDir = "/opt/DVDMaker";
	public static final String defaultDir = "/home";
	public static final int numAudioFiles = 1;
	
	public static void main(String args[]) {
		new DVDMaker();
	}
	
	public DVDMaker() {
		this(true);
	}
	
	public DVDMaker(boolean pal) {
		super("DVD Maker");
		this.pal = pal;
		JPanel content = (JPanel) this.getContentPane();
		canvas = new ImagePanel();
		content.add(canvas,"Center");
		JPanel topRow = new JPanel();
		JPanel titlesPanel = new JPanel();
		JPanel navAndTitles=new JPanel();
		navAndTitles.setLayout(new BorderLayout());
		navAndTitles.add(topRow,"North");
		navAndTitles.add(titlesPanel,"South");
		JPanel botRow = new JPanel();
		content.add(navAndTitles,"North");
		content.add(botRow,"South");
		jfc = new JFileChooser();
		jfc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		jfc.setDialogTitle("Please Choose a Folder to Convert");
		jfc.setCurrentDirectory(new File(defaultDir));
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		topRow.add(ppprevBut);
		topRow.add(prevBut);
		topRow.add(nextBut);
		topRow.add(nnnextBut);

		titlesPanel.add(addTitle);
		titlesPanel.add(titleLabel);
		titlesPanel.add(delTitle);

		botRow.add(doShow);
		prevBut.addActionListener(this);
		nextBut.addActionListener(this);
		addTitle.addActionListener(this);
		delTitle.addActionListener(this);
		doShow.addActionListener(this);
		
		
		
		this.pack();
		this.setVisible(true);
	
		// do last!
		pickFolder();
		
	}
	boolean pal;
	ImagePanel canvas;
	JFileChooser jfc;
	File workFolder;
	JButton nextBut = new JButton("Next >");
	JButton prevBut = new JButton("< Back");
	JButton nnnextBut = new JButton(">>>>");
	JButton ppprevBut = new JButton("<<<<");
	JButton addTitle = new JButton("Add Title Slide");
	JLabel titleLabel = new JLabel("<No Title Slide>");
	JButton delTitle = new JButton("Delete Title Slide");
	JButton doShow = new JButton("Generate Show");
	File[] files;
	String[] titles;
	private int imageNo=0;
	ArrayList<String> createdFiles;
	
	
	public void pickFolder() {
		int response;
		do {
			response = jfc.showOpenDialog(this);
			if (response != JFileChooser.APPROVE_OPTION) {
				int quit = JOptionPane.showConfirmDialog(this,"You should choose a folder, unless you want to quit. Do you want to quit??","Error!",JOptionPane.YES_NO_OPTION,JOptionPane.ERROR_MESSAGE);
				if (quit == JOptionPane.YES_OPTION) {
					System.exit(0);
				}
			}
		} while (response != JFileChooser.APPROVE_OPTION);
		workFolder = jfc.getSelectedFile();
		files = workFolder.listFiles(new MoviesAndImagesFilter());
//		for (int i=0; i<files.length; i++) {
//			System.out.println(files[i].toString());
//		}
		titles = new String[files.length];
		for (int i=0; i<files.length; i++) {
			for (int j=0; j<files.length-1; j++) {
				if (files[j].getName().compareTo(files[j+1].getName()) > 0) {
					File tmp = files[j];
					files[j] = files[j+1];
					files[j+1]=tmp;
				}			
			}
			
		}
		setImage();
	}

	public void actionPerformed(ActionEvent e) {
		JButton src = (JButton) e.getSource();
		if (src == nextBut) {
			changeImage(1);
		} else if (src == prevBut) {
			changeImage(-1);
		} else if (src == nnnextBut) {
			changeImage(5);
		} else if (src == ppprevBut) {
			changeImage(-5);
		} else if (src == addTitle) {
			String text = JOptionPane.showInputDialog(this,"Title Slide Text to insert before this image/movie :",(titles[imageNo] == null ? "" : titles[imageNo]));
			titles[imageNo] = text;
			updateTitle();
		} else if (src == delTitle) {
			titles[imageNo] = null;
			updateTitle();
		} else if (src == doShow) {
			int response = JOptionPane.showConfirmDialog(this,"Are you sure you are ready to generate the DVD?","Are you sure?",JOptionPane.YES_NO_CANCEL_OPTION);
			if (response == JOptionPane.YES_OPTION) {
				createdFiles = new ArrayList<String>();
				String dvdTitle = JOptionPane.showInputDialog(this,"What do you want to call your DVD?","Our Photos");
				File script = new File(workFolder,"makeDVD.sh");
				try {
					PrintWriter pw = new PrintWriter(new FileWriter(script));
					createdFiles.add(script.getAbsolutePath());
					PrintWriter dvdSlide = null;
					int count = 0;
					boolean infile = false;
					for (int i=0; i<files.length; i++) {
						// look through the files one by one...
						if (titles[i] != null) {
							// perhaps a title has been defined....
							String titleFile = createTitleSlide(pw,i);
							if (!infile) {
								infile = true;
								dvdSlide = startSlideShow(count);
							} else {
								dvdSlide.println("crossfade:1:");
							}
							dvdSlide.println(titleFile+":5:::");
						}
						if (MoviesAndImagesFilter.isImage(files[i].getAbsolutePath())) {
							// or perhaps the next file is an image file...
							if (!infile) {
								infile = true;
								dvdSlide = startSlideShow(count);
							} else {
								dvdSlide.println("crossfade:1:");
							}
							dvdSlide.println(files[i].getAbsolutePath()+":5:::");
						} else { // it's a movie
							if (infile) {
								dvdSlide.println("fadeout:1:");
								dvdSlide.close();
								pw.print("dvd-slideshow -n chapter"+count+" ");
								if (pal) { 
									pw.print("-p ");
								}
								pw.println("scene"+count+".txt");
								createdFiles.add(new File(workFolder,"chapter"+count+".xml").getAbsolutePath());
								createdFiles.add(new File(workFolder,"chapter"+count+".vob").getAbsolutePath());
								count++;
							}
							infile = false;
							createMovieXML(count);
							pw.println("mencoder -oac lavc -ovc lavc -of mpeg -mpegopts format=dvd -vf scale=720:"+(pal?576:480)+",harddup "
							+ "-srate 48000 -af lavcresample=48000 " 
							+ " -lavcopts vcodec=mpeg2video:vrc_buf_size=1835:vrc_maxrate=9800:vbitrate=5000:keyint="+(pal?15:18)+":aspect=4/3:"
							+ "acodec=ac3:abitrate=192 -ofps "+(pal?"25":"30000/1001")+" -o chapter"+count+".vob "+files[i].getAbsolutePath());
							createdFiles.add(new File(workFolder,"chapter"+count+".vob").getAbsolutePath());
							count++;
						}
					}
					if (infile) {
						// close the last slideshow sequence...
						dvdSlide.println("fadeout:1:");
						dvdSlide.close();
						pw.println("dvd-slideshow -n chapter"+count+(pal?" -p":"")+" scene"+count+".txt");
						createdFiles.add(new File(workFolder,"chapter"+count+".xml").getAbsolutePath());
						createdFiles.add(new File(workFolder,"chapter"+count+".vob").getAbsolutePath());
					}
					// now, combine the chapters into a dvd
					pw.print("dvd-menu ");
					for (int i=0; i<=count; i++) {
						pw.print("-t \"Scene "+(i+1)+"\" " );
					}
					for (int i=0; i<=count; i++) {
						pw.print("-f chapter"+i+".vob ");
					}
//					pw.print("-o "+new File(workFolder,"DVDfs").getAbsolutePath()+" ");
					pw.print("-n '"+dvdTitle+"' ");
					pw.print("-b "+installDir+"/titlebg.jpg ");
					pw.print("-a "+installDir+"/audio"+(int)((Math.random()*numAudioFiles))+".ogg ");
					if (pal) pw.print("-p ");
					pw.println("-c -fadein -fadeout");
					pw.println("mkdir dvd_fs/images");
					for (File f : files) {
						if (MoviesAndImagesFilter.isImage(f.getName())) {
							pw.println("cp "+f.getAbsolutePath()+" dvd_fs/images");
						}
					}
					pw.println("mplayer -sid 0 dvd:// -dvd-device "+new File(workFolder,"dvd_fs").getAbsolutePath());
					pw.println("read -p \"Ready to burn to DVD. Press any key! (or CTRL-C to stop here)\"");
					pw.println("growisofs -dvd-compat -Z /dev/hdc -dvd-video ./dvd_fs/");
					pw.println("read -p \"Ready to clean up. Press any key! (or CTRL-C to stop here)\"");
					createdFiles.add(new File(workFolder,"dvd-menu.log").getAbsolutePath());
					createdFiles.add(new File(workFolder,"dvd-slideshow.log").getAbsolutePath());
					createdFiles.add(new File(workFolder,"fadein.vob").getAbsolutePath());
					createdFiles.add(new File(workFolder,"fadeout.vob").getAbsolutePath());
					createdFiles.add(new File(workFolder,"menu.vob").getAbsolutePath());
					createdFiles.add(new File(workFolder,"vmgm.xml").getAbsolutePath());
					for (String s : createdFiles) {
						pw.println("rm -f "+s);
					}
					pw.println("read -p \"If you want to keep the DVD folder, press CTRL-C NOW! (otherwise, press any key)\"");
					pw.println("rm -rf dvd_fs");
					pw.close();
					
					JOptionPane.showMessageDialog(this,"The script to create the DVD, and its supporting files,\n has been written to "+script,"Done!",JOptionPane.INFORMATION_MESSAGE);
				} catch (IOException e1) {
					JOptionPane.showMessageDialog(this,"Oh dear! An IO Error occurred!","Error!",JOptionPane.ERROR_MESSAGE);
				}
			}
		}
		
	}

	private void createMovieXML(int count) throws IOException {
		PrintWriter pw = new PrintWriter(new FileWriter(new File(workFolder,"chapter"+count+".xml")));
		createdFiles.add(new File(workFolder,"chapter"+count+".xml").getAbsolutePath());
		pw.println("<dvdauthor>");
		pw.println("  <vmgm />");
		pw.println("    <titleset>");
		pw.println("      <titles>");
		pw.println("        <pgc>");
		pw.println("          <vob file=\"chapter"+count+".mpg\"/>");
		pw.println("        </pgc>");
		pw.println("      </titles>");
		pw.println("    </titleset>");
		pw.println("</dvdauthor>");
		pw.close();
	}

	private PrintWriter startSlideShow(int count) throws IOException {
		PrintWriter rtn = new PrintWriter(new FileWriter(new File(workFolder,"scene"+count+".txt")));
		createdFiles.add(new File(workFolder,"scene"+count+".txt").getAbsolutePath());
		rtn.println(installDir+"/audio"+(int)((Math.random()*numAudioFiles))+".ogg:1:fadein:2:fadeout:2");
		return rtn;
	}

	private String createTitleSlide(PrintWriter pw, int i) {
		pw.println("convert -font "+installDir+"/titlefont.ttf -pointsize 144 -gravity Center -fill yellow -draw \"text 0,-2 '"+titles[i]+"'\"  "+installDir+"/titlebg.jpg "+new File(workFolder,"DVDtitle"+i+".jpeg").getAbsolutePath());
		createdFiles.add(new File(workFolder,"DVDtitle"+i+".jpeg").getAbsolutePath());
		return new File(workFolder,"DVDtitle"+i+".jpeg").getAbsolutePath();
	}

	private void changeImage(int i) {
		imageNo += i;
		if (imageNo < 0) imageNo = 0;
		if (imageNo >= files.length) {
			imageNo = files.length-1;
		}
		setImage();
	}
	
	public void setImage() {
		if (MoviesAndImagesFilter.isImage(files[imageNo].getAbsolutePath())) {
			canvas.setImage(files[imageNo]);
		} else {
			canvas.setImage(new File(installDir+"/movie.png"));
		}
		updateTitle();
	}
	
	public void updateTitle() {
		if (titles[imageNo] == null) {
			titleLabel.setText("<No Title Slide>");
		} else {
			titleLabel.setText(titles[imageNo]);
		}
	}
}
